// Copyright (C) 2013 by FZI Forschungszentrum Informatik am Karlsruher Institut fuer Technologie
// 
// Author: Benjamin Ranft (benjamin.ranft@web.de)
// 
// This file is part of demoHeterogeneousParallelism.
// 
// demoHeterogeneousParallelism is free software: you can redistribute it and/or modify it under the terms of the GNU 
// General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your 
// option) any later version.
// 
// demoHeterogeneousParallelism is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
// even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
// for more details.
// 
// You should have received a copy of the GNU General Public License along with demoHeterogeneousParallelism. If not, 
// see <http://www.gnu.org/licenses/>.


// place-holder for your code: GEMM sample application
// ===================================================

#pragma once

#include "hawaii/GPU/openCV.h"
#include "hawaii/common/hardware.h"
#ifndef HAWAII_NO_CUDA
	#include <cublas_v2.h>
#else
	typedef void* cublasHandle_t ;
#endif

// scaled matrix multiplication: result = A * B * alpha
void multiplyCPU( const cv::Mat          A,
                  const cv::Mat          B,
                        cv::Mat&         result ) ;
void multiplyGPU( const cv::gpu::GpuMat  A,
                  const cv::gpu::GpuMat  B,
                        cv::gpu::GpuMat& result,
                        cublasHandle_t   handle = nullptr,
                        cv::gpu::Stream& stream = cv::gpu::Stream::Null() ) ;

// scaled matrix addition: result = AB * alpha + C * beta
void addScaledCPU( const cv::Mat          AB,
                   const cv::Mat          C,
                         cv::Mat&         result,
                   const float            alpha  = 1.0,
                   const float            beta   = 1.0 ) ;
void addScaledGPU( const cv::gpu::GpuMat  AB,
                   const cv::gpu::GpuMat  C,
                         cv::gpu::GpuMat& result,
                   const float            alpha  = 1.0,
                   const float            beta   = 1.0,
                         cv::gpu::Stream& stream = cv::gpu::Stream::Null() ) ;

// general matrix-multiply (GEMM): result = A * B * alpha + C * beta
void gemmCPU( const cv::Mat          A,
              const cv::Mat          B,
              const cv::Mat          C,
                    cv::Mat&         result,
              const float            alpha  = 1.0,
              const float            beta   = 1.0 ) ;
void gemmGPU( const cv::gpu::GpuMat  A,
              const cv::gpu::GpuMat  B,
              const cv::gpu::GpuMat  C,
                    cv::gpu::GpuMat& result,
              const float            alpha  = 1.0,
              const float            beta   = 1.0,
                    cublasHandle_t   handle = nullptr,
                    cv::gpu::Stream& stream = cv::gpu::Stream::Null() ) ;
