# Copyright (C) 2013 by FZI Forschungszentrum Informatik am Karlsruher Institut fuer Technologie
# 
# Author: Benjamin Ranft (benjamin.ranft@web.de)
# 
# This file is part of libHawaii.
# 
# libHawaii is free software: you can redistribute it and/or modify it under the terms of the GNU General Public 
# License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later 
# version.
# 
# libHawaii is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along with libHawaii. If not, see 
# <http://www.gnu.org/licenses/>.


# Overview
# ========
# Each project related to libHawaii has an own "config.mk" describing its dependencies. In contrast, compiler flags for 
# the three build modes "debug", "profile" and "release" are configured centrally in "libHawaii/rules.mk".


# Build artifact, references from/to other projects
# =================================================
# easily integrate with other projects related to libHawaii

# push relative path from originally called "makefile" to this "config.mk"
MAKEFILE_STACK += $(lastword $(MAKEFILE_LIST))

# un-comment any libraries linked by this project
#include ../libHawaii/config.mk # heterogeneous auto-tuned data-element-, data-flow- and task-parallelism

# artifact to be built in this project: "0" builds an executable, "1" a static library and "2" a shared library.
ART_TYPE := 1
# names for each build-mode
ART_NAME_DBG := HawaiiDbg
ART_NAME_PRF := HawaiiPrf
ART_NAME_REL := Hawaii

# if referenced by another project, add this one's headers and libraries (if applicable)
REL_DIR := $(strip $(dir $(lastword $(MAKEFILE_STACK))))
ifneq ($(REL_DIR),./)
   INC_DIRS += -I$(REL_DIR)inc
   ifeq ($(ART_TYPE),1)
      LIB_DIRS_ALL += -L$(REL_DIR)lib
      LIBS_DBG += -l:lib$(ART_NAME_DBG).a
      LIBS_PRF += -l:lib$(ART_NAME_PRF).a
      LIBS_REL += -l:lib$(ART_NAME_REL).a
      LIBDEP_DBG += $(REL_DIR)lib/lib$(ART_NAME_DBG).a
      LIBDEP_PRF += $(REL_DIR)lib/lib$(ART_NAME_PRF).a
      LIBDEP_REL += $(REL_DIR)lib/lib$(ART_NAME_REL).a
   endif
   ifeq ($(ART_TYPE),2)
      LIB_DIRS_ALL += -L$(REL_DIR)lib -Wl,-rpath=$(REL_DIR)lib
      LIBS_DBG += -l:lib$(ART_NAME_DBG).so
      LIBS_PRF += -l:lib$(ART_NAME_PRF).so
      LIBS_REL += -l:lib$(ART_NAME_REL).so
   endif
endif

# pop relative path from originally called "makefile" to this "config.mk"
MAKEFILE_STACK := $(filter-out $(lastword $(MAKEFILE_STACK)), $(MAKEFILE_STACK))


# Dependencies
# ============
# configure libraries to be included and linked below - you may append(!) to the following variables:
# - "INC_DIRS" for include directories
# - "DEFS" for defines
# - "LIB_DIRS_{ALL,DBG,PRF,REL,X86,A64,X86_DBG,X86_PRF,X86_REL,A64_DBG,A64_PRF,A64_REL}, depending on architecture and 
#   build mode, for library directories
# - "LIBS_{ALL,DBG,PRF,REL,X86,A64,X86_DBG,X86_PRF,X86_REL,A64_DBG,A64_PRF,A64_REL} for libraries

# Boost multi-purpose libraries
# user note: Versions 1.46.0 and 1.54.0 (used by Ubuntu 12.04 LTS and 14.04 LTS respectively) are supported, but other 
#            more recent versions should work as well.
BOOST_DIR := 
ifneq ($(BOOST_DIR),)
   INC_DIRS     += -I$(BOOST_DIR)/include
   LIB_DIRS_ALL += -L$(BOOST_DIR)/lib -Wl,-rpath=$(BOOST_DIR)/lib
endif
#LIBS_ALL += -lboost_atomic               # only available since version 1.53.0!
#LIBS_ALL += -lboost_chrono
#LIBS_ALL += -lboost_context              # only available since version 1.51.0!
#LIBS_ALL += -lboost_coroutine            # only available since version 1.54.0!
#LIBS_ALL += -lboost_date_time
#LIBS_ALL += -lboost_exception
LIBS_ALL += -lboost_filesystem
#LIBS_ALL += -lboost_graph
#LIBS_ALL += -lboost_iostreams
#LIBS_ALL += -lboost_locale               # only available since version 1.48.0!
#LIBS_ALL += -lboost_log                  # only available since version 1.54.0!
#LIBS_ALL += -lboost_log_setup            # only available since version 1.54.0!
#LIBS_ALL += -lboost_math_c99
#LIBS_ALL += -lboost_math_c99f
#LIBS_ALL += -lboost_math_c99l
#LIBS_ALL += -lboost_math_tr1
#LIBS_ALL += -lboost_math_tr1f
#LIBS_ALL += -lboost_math_tr1l
#LIBS_ALL += -lboost_prg_exec_monitor
#LIBS_ALL += -lboost_program_options
#LIBS_ALL += -lboost_python
#LIBS_ALL += -lboost_random
#LIBS_ALL += -lboost_regex
#LIBS_ALL += -lboost_serialization
#LIBS_ALL += -lboost_signals
LIBS_ALL += -lboost_system
#LIBS_ALL += -lboost_test_exec_monitor
LIBS_ALL += -lboost_thread
#LIBS_ALL += -lboost_timer               # only available since version 1.48.0!
#LIBS_ALL += -lboost_unit_test_framework
#LIBS_ALL += -lboost_wave
#LIBS_ALL += -lboost_wserialization

# OpenCV image processing library
# user note: Currently, both the stable "2.4" branch and - with some delay - the "master" branch from "github.com/
#            Itseez/opencv/branches" are supported. I do not recommend the packages from your Linux distribution 
#            because they may lack GPU support and SIMD optimizations.
# developer note: The GPU/CUDA module has been renamed and split between versions "2.4" and "master", which makes the 
#           conditions below necessary.
OPENCV_DIR := 
ifeq ($(OPENCV_DIR),)
   OPENCV_DIR := $(wildcard /opt/ros/indigo)
endif
ifeq ($(OPENCV_DIR),)
   OPENCV_DIR := $(wildcard /opt/ros/hydro)
endif
ifneq ($(OPENCV_DIR),)
   INC_DIRS     += -I$(OPENCV_DIR)/include
   LIB_DIRS_ALL += -L$(OPENCV_DIR)/lib -Wl,-rpath=$(OPENCV_DIR)/lib
endif
ifneq ($(wildcard $(OPENCV_DIR)/lib/libopencv_bioinspired.so)$(shell ldconfig -p | grep libopencv_bioinspired.so),)
   #LIBS_ALL += -lopencv_bioinspired
endif
LIBS_ALL += -lopencv_calib3d
#LIBS_ALL += -lopencv_contrib
LIBS_ALL += -lopencv_core
ifneq ($(wildcard $(OPENCV_DIR)/lib/libopencv_cuda.so)$(shell ldconfig -p | grep libopencv_cuda.so),)
   LIBS_ALL += -lopencv_cuda
endif
ifneq ($(wildcard $(OPENCV_DIR)/lib/libopencv_cudaarithm.so)$(shell ldconfig -p | grep libopencv_cudaarithm.so),)
   LIBS_ALL += -lopencv_cudaarithm
endif
ifneq ($(wildcard $(OPENCV_DIR)/lib/libopencv_cudabgsegm.so)$(shell ldconfig -p | grep libopencv_cudabgsegm.so),)
   #LIBS_ALL += -lopencv_cudabgsegm
endif
ifneq ($(wildcard $(OPENCV_DIR)/lib/libopencv_cudacodec.so)$(shell ldconfig -p | grep libopencv_cudacodec.so),)
   #LIBS_ALL += -lopencv_cudacodec
endif
ifneq ($(wildcard $(OPENCV_DIR)/lib/libopencv_cudafeatures2d.so)$(shell ldconfig -p | grep libopencv_cudafeatures2d.so),)
   LIBS_ALL += -lopencv_cudafeatures2d
endif
ifneq ($(wildcard $(OPENCV_DIR)/lib/libopencv_cudafilters.so)$(shell ldconfig -p | grep libopencv_cudafilters.so),)
   LIBS_ALL += -lopencv_cudafilters
endif
ifneq ($(wildcard $(OPENCV_DIR)/lib/libopencv_cudaimgproc.so)$(shell ldconfig -p | grep libopencv_cudaimgproc.so),)
   LIBS_ALL += -lopencv_cudaimgproc
endif
ifneq ($(wildcard $(OPENCV_DIR)/lib/libopencv_cudalegacy.so)$(shell ldconfig -p | grep libopencv_cudalegacy.so),)
   #LIBS_ALL += -lopencv_cudalegacy
endif
ifneq ($(wildcard $(OPENCV_DIR)/lib/libopencv_cudaoptflow.so)$(shell ldconfig -p | grep libopencv_cudaoptflow.so),)
   #LIBS_ALL += -lopencv_cudaoptflow
endif
ifneq ($(wildcard $(OPENCV_DIR)/lib/libopencv_cudastereo.so)$(shell ldconfig -p | grep libopencv_cudastereo.so),)
   LIBS_ALL += -lopencv_cudastereo
endif
ifneq ($(wildcard $(OPENCV_DIR)/lib/libopencv_cudawarping.so)$(shell ldconfig -p | grep libopencv_cudawarping.so),)
   LIBS_ALL += -lopencv_cudawarping
endif
ifneq ($(wildcard $(OPENCV_DIR)/lib/libopencv_cudev.so)$(shell ldconfig -p | grep libopencv_cudev.so),)
   #LIBS_ALL += -lopencv_cudev
endif
LIBS_ALL += -lopencv_features2d
#LIBS_ALL += -lopencv_flann
ifneq ($(wildcard $(OPENCV_DIR)/lib/libopencv_gpu.so)$(shell ldconfig -p | grep libopencv_gpu.so),)
   LIBS_ALL += -lopencv_gpu
endif
LIBS_ALL += -lopencv_highgui
LIBS_ALL += -lopencv_imgproc
#LIBS_ALL += -lopencv_java
#LIBS_ALL += -lopencv_legacy
#LIBS_ALL += -lopencv_ml
#LIBS_ALL += -lopencv_nonfree
LIBS_ALL += -lopencv_objdetect
#LIBS_ALL += -lopencv_ocl
ifneq ($(wildcard $(OPENCV_DIR)/lib/libopencv_optim.so)$(shell ldconfig -p | grep libopencv_optim.so),)
   #LIBS_ALL += -lopencv_optim
endif
#LIBS_ALL += -lopencv_photo
ifneq ($(wildcard $(OPENCV_DIR)/lib/libopencv_shape.so)$(shell ldconfig -p | grep libopencv_shape.so),)
   #LIBS_ALL += -lopencv_shape
endif
ifneq ($(wildcard $(OPENCV_DIR)/lib/libopencv_softcascade.so)$(shell ldconfig -p | grep libopencv_softcascade.so),)
   #LIBS_ALL += -lopencv_softcascade
endif
#LIBS_ALL += -lopencv_stitching
#LIBS_ALL += -lopencv_superres
#LIBS_ALL += -lopencv_ts
LIBS_ALL += -lopencv_video
#LIBS_ALL += -lopencv_videostab
#LIBS_ALL += -lopencv_world

# CUDA GPU computing library (optional)
# user note: Version 5.5 (used by Ubuntu 14.04 LTS) and the current release from "developer.nvidia.com/cuda-downloads" 
#            is tested and supported, older ones may or may not work as well.
# developer note: For convenience, one does not need to disable CUDA manually if NVCC cannot be found anyway.
CUDA_USE := 1
CUDA_DIR         := 
CUDA_SAMPLES_DIR := 
ifeq ($(CUDA_DIR),)
   CUDA_DIR := $(wildcard /usr/local/cuda)
endif
ifeq ($(CUDA_SAMPLES_DIR),)
   ifeq ($(CUDA_DIR),)
      CUDA_SAMPLES_DIR := $(wildcard /usr/local/cuda/samples)
   else
      CUDA_SAMPLES_DIR := $(wildcard $(CUDA_DIR)/samples)
   endif
endif
ifneq ($(CUDA_DIR),)
   INC_DIRS     += -I$(CUDA_DIR)/include
   LIB_DIRS_X86 += -L$(CUDA_DIR)/lib   -Wl,-rpath=$(CUDA_DIR)/lib
   LIB_DIRS_A64 += -L$(CUDA_DIR)/lib64 -Wl,-rpath=$(CUDA_DIR)/lib64
endif
ifneq ($(CUDA_SAMPLES_DIR),)
   INC_DIRS += -I$(CUDA_SAMPLES_DIR)/common/inc
endif
ifneq ($(CUDA_USE),0)
ifneq ($(wildcard $(CUDA_DIR)/bin/nvcc)$(shell which nvcc),)
   LIBS_ALL += -lcublas
   LIBS_ALL += -lcudart
#   LIBS_ALL += -lcufft
#   LIBS_ALL += -lcufftw
#   LIBS_X86 += -lcuinj32
#   LIBS_A64 += -lcuinj64
#   LIBS_ALL += -lcurand
#   LIBS_ALL += -lcusparse
   ifneq ($(wildcard $(CUDA_DIR)/lib*/libnpp.so)$(shell ldconfig -p | grep libnpp.so),) # CUDA <= 5.0
      LIBS_ALL += -lnpp
   endif
   ifneq ($(wildcard $(CUDA_DIR)/lib*/libnppc.so)$(shell ldconfig -p | grep libnppc.so),) # CUDA >= 5.5
      LIBS_ALL += -lnppc
   endif
   ifneq ($(wildcard $(CUDA_DIR)/lib*/libnppi.so)$(shell ldconfig -p | grep libnppi.so),) # CUDA >= 5.5
      LIBS_ALL += -lnppi
   endif
   ifneq ($(wildcard $(CUDA_DIR)/lib*/libnpps.so)$(shell ldconfig -p | grep libnpps.so),) # CUDA >= 5.5
      LIBS_ALL += -lnpps
   endif
#   LIBS_ALL += -lnvToolsExt
else
   CUDA_USE := 0
   DEFS += -DHAWAII_NO_CUDA
endif
else
   DEFS += -DHAWAII_NO_CUDA
endif

