// Copyright (C) 2013 by FZI Forschungszentrum Informatik am Karlsruher Institut fuer Technologie
// 
// Author: Benjamin Ranft (benjamin.ranft@web.de)
// 
// This file is part of libHawaii.
// 
// libHawaii is free software: you can redistribute it and/or modify it under the terms of the GNU General Public 
// License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later 
// version.
// 
// libHawaii is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
// warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License along with libHawaii. If not, see 
// <http://www.gnu.org/licenses/>.


// item to contain named "hawaii::AutoMat" images
// ==============================================

#pragma once

#include "hawaii/flow/item.h"
#include "hawaii/GPU/autoMat.h"
#include <map>

namespace hawaii {
namespace flow {

// forward declaration
class ItemNamedImages ;

// additional information for partitioning images images into horizontal stripes
class PartInfoImageOverlap : public PartInfoBase {
	
	// pre-allocation: To avoid deep image copies while merging, you may optionally define whole images to be allocated 
	//                 before splitting. Their respective size and type are determined from a reference image. However, 
	//                 the type may be overridden and the size may be reduced by specifying the clipping of window-
	//                 based processing steps.
	public:
	static const int      typeSame   ;
	static const cv::Size windowNone ;
	void addPreAlloc( const std::string nameArg,
	                  const std::string nameRefArg,
	                  const int         typeArg    = typeSame,
	                  const cv::Size    winArg     = windowNone ) {
		PreAlloc preAlloc ;
		preAlloc.name    = nameArg    ;
		preAlloc.nameRef = nameRefArg ;
		preAlloc.type = typeArg ;
		preAlloc.win  = winArg  ;
		this->preAllocs.push_back( preAlloc ) ;
	}
	struct PreAlloc {
		std::string name,
		            nameRef ;
		int      type ;
		cv::Size win  ;
	} ;
	std::vector< PreAlloc > preAllocs ;
	
	// partitioning: If the final result to be merged has not been pre-allocated, you must specify the overlap required 
	//               by window-based processing steps. It is by default applied to the image with the least rows, but 
	//               you may override the name of this reference image.
	public:
	PartInfoImageOverlap( const int         winHeightArg = 1,
	                      const std::string nameRefArg   = std::string() ) :
		winHeight( winHeightArg ),
		nameRef(   nameRefArg   ) {
	}
	const int         winHeight ;
	const std::string nameRef   ;
} ;

// container for pairs of names and images
class ItemNamedImages : public ItemTemplate< ItemNamedImages, PartInfoImageOverlap > {
	
	// read from or assign to "myItem.images[ "Lena, the image processing example" ] ;"
	// user note: Assigning to a new name inserts, to an existing one overwrites.
	public:
	std::map< const std::string, hawaii::AutoMat > images ;
	
	// require running number to be set
	public:
	ItemNamedImages( const uint64_t serialNumberArg,
	                 const double   ratioArg        = 1.0 ) :
		ItemTemplate< ItemNamedImages, PartInfoImageOverlap >( serialNumberArg, ratioArg ) {
	}
	
	// split into or merge horizontal image stripes
	// user note: Splitting affects any existing CPU and GPU memory, but merging takes place in CPU memory only.
	protected:
	int rowsRef ;
	public:
	virtual void splitImpl(       PtrVec&                parts,
	                        const std::vector< double >& ratios,
	                        const PartInfoImageOverlap&  partInfo = PartInfoImageOverlap() ) override ;
	virtual void mergeImpl( const PtrVec&               parts,
	                              Ptr&                  whole,
	                        const PartInfoImageOverlap& partInfo = PartInfoImageOverlap() ) override ;
} ;

} } // namespaces "hawaii::flow"
