// Copyright (C) 2013 by FZI Forschungszentrum Informatik am Karlsruher Institut fuer Technologie
// 
// Author: Benjamin Ranft (benjamin.ranft@web.de)
// 
// This file is part of libHawaii.
// 
// libHawaii is free software: you can redistribute it and/or modify it under the terms of the GNU General Public 
// License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later 
// version.
// 
// libHawaii is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
// warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License along with libHawaii. If not, see 
// <http://www.gnu.org/licenses/>.


// classical pipeline
// ==================

#include "hawaii/flow/pipeline.h"

namespace hawaii {
namespace flow {

// "Pipeline": wrapper around consecutive flow components
// ------------------------------------------------------

// allow the variadic template to terminate
void Pipeline::add() {
	HAWAII_ERROR_CONDITIONAL( this->flows.size() > 0 && ( this->flows.size() - this->queues.size() ) != 1,
	                          "invalid initialization arguments" ) ;
}

// reset internal state
void Pipeline::reset() {
	for( auto& flow : this->flows ) { flow->reset() ; }
	FlowBase::reset() ;
}

// run this pipeline, wait for it to finish or directly stop it
void Pipeline::run()  { this->stop() ;
                        this->reset() ;
                        for( FlowBase::Ptr& flow : this->flows ) { flow->run()  ; } }
void Pipeline::wait() { for( FlowBase::Ptr& flow : this->flows ) { flow->wait() ; } }
void Pipeline::stop() { for( FlowBase::Ptr& flow : this->flows ) { flow->stop() ; } }

// estimate power consumption
double Pipeline::power() const {
	double ret = 0.0 ;
	for( const auto& flow : this->flows ) {
		ret += flow->power() ;
	}
	return ret ;
}

// print evaluation
void Pipeline::eval( size_t indentationName,
	                  size_t indentationData ) const {
	// own statistics
	FlowBase::eval( indentationName, indentationData ) ;
	
	// find longest name of wrapped flow components
	size_t nameSizeMax = 0 ;
	for( const auto& flow : this->flows ) {
		nameSizeMax = std::max( nameSizeMax, flow->name.size() ) ;
	}
	
	// print their statistics
	for( const auto& flow : this->flows ) {
		flow->eval( indentationName + 1, nameSizeMax ) ;
	}
}

} } // namespaces "hawaii::flow"
